;; Generated package description from vigenere.el  -*- no-byte-compile: t -*-
(define-package "vigenere" "1.0" "Run a vigenere cipher on a block of text ;" '((emacs "25.1")) :commit "434270403845789f4be32102c573ea965e870d19" :authors '(("Ian Dunn" . "dunni@gnu.org")) :maintainer '("Ian Dunn" . "dunni@gnu.org") :keywords '("data" "vigenere" "cipher") :url "https://elpa.gnu.org/packages/vigenere.html")
